
  FUNCTION KERNEL, X, type=TYPE, limits=LIMITS
; ===========================================================
; Returns the value of a KERNEL (=weight, window) function.
;
; Input:
;   X         :  Argument of the kernel function.
;   TYPE      :  Type of the kernel function.
;                Default is 'rectangular'.
;                Other choices:
;                'triangular', 'gaussian'.
;
; Output:
;   on return :  Result of the kernel function at X.
;   LIMITS    :  If keyword is set, LIMITS contains
;                a two-element array with the effective
;                limits of the kernel function.
;
; JoV, 7/2003. 
; =========================================================== 

; Check keyword settings. 
; -----------------------
  if not keyword_set(type) then TYPE='rectangular'

; Output should be of the same (array) type as input. 
; ---------------------------------------------------
  RESULT = X
  RESULT[*] = 0.

; Compute kernel function of argument X. 
; --------------------------------------
  case TYPE of
; ................. rectangular kernel for naive estimator 
    'rectangular' : begin
                    if keyword_set(LIMITS) then LIMITS = [-1.,1.]
                    nz = where( abs(X) le 1. , cnz )
                    if (cnz ne 0) then RESULT[nz] = 0.5 
                    end
; ................. triangular kernel 
    'triangular'  : begin
                    if keyword_set(LIMITS) then LIMITS = [-1.,1.]
                    neg = where( (X gt -1.) and (X le 0.) , cneg )
                    pos = where( (X ge  0.) and (X lt 1.) , cpos )
                    if (cneg ne 0) then RESULT[neg] = 1.+X[neg]
                    if (cpos ne 0) then RESULT[pos] = 1.-X[pos]
                    end
; ................. gaussian kernel can be truncated for X>6
    'gaussian'    : begin
                    if keyword_set(LIMITS) then LIMITS = [-6., 6.]
                    nz = where( abs(X) le 6. , cnz )
                    if (cnz ne 0)     $
                      then RESULT[nz] = exp(-X[nz]^2/2.)/2.5066283
                    end
; ................. error in specification of kernel type 
    else          : begin 
                    PRINT, '*** ERROR in function KERNEL ***'
                    PRINT, '*** Non-supported kernel type ***'
                    STOP
                    end
  endcase

; All done, return
; ----------------
  return, RESULT
  end
