
  FUNCTION PDFEST, U, DATA, wintype=WINTYPE, winwidth=WINWIDTH
; ==============================================================
; Estimates the pdf of a data set DATA at argument U using the
; kernel method.
; 
; Input:
;   U         :  Argument of the pdf. 
;   DATA      :  Data set with samples from the pdf.
;   WINTYPE   :  Type of the window function used in the
;                kernel method.  Default is 'rectangular'.
;                Other choices: 'triangular', 'gaussian' .
;   WINWIDTH  :  Width of the window function.  Default: 1.
;
; Output:
;   on return :  Estimate of the pdf at argument U.
;                If U is an array, an array of the same type 
;                will be returned. 
;
; JoV, 7/2003. 
; ==============================================================

; Check keyword settings. 
; -----------------------
  if not keyword_set(wintype) then WINTYPE='rectangular'
  if not keyword_set(winwidth) then WINWIDTH=1. 

; Initialize RESULT to be of the same type as the input U. 
; --------------------------------------------------------
  RESULT = U
  RESULT[*] = 0. 
  nu = n_elements(U)

; -----------------------------
; Loop over all array elements. 
; -----------------------------
  for i=0,nu-1 do begin 

; Prepare kernel evaluation. 
; --------------------------
; ... set effective limits
  case WINTYPE of
    'rectangular' : limits = [-1.,1.]
    'triangular'  : limits = [-1.,1.]
    'gaussian'    : limits = [-6.,6.]
    else          : begin 
                    PRINT, '*** ERROR in function PDFEST ***'
                    PRINT, '*** Non-supported value of WINTYPE ***'
                    STOP
                    end
  endcase
; ... find index values which give non-zero results 
  arg = (U[i]-DATA) / WINWIDTH
  nonzero = where( (arg ge limits[0]) and (arg le limits[1]), $ 
                   count )

; Compute pdf estimate. 
; ---------------------
  if ( count ne 0 )                                             $
    then RESULT[i] = total( KERNEL(arg[nonzero],type=WINTYPE) ) $
                      / (n_elements(DATA)*WINWIDTH)             $
    else RESULT[i] = 0. 

; ------------
; End of loop.
; ------------
  endfor

; All done, return.
; ----------------- 
  return, RESULT
  end

