
  PRO GETTBR, T, B, R, RBAR, DATABAD,                            $
              filenames=FILENAMES, tbeg=TBEG, tend=TEND, dt=DT,  $
              time_format=TIME_FORMAT
; =====================================================================
; GETTBR reads data magnetic field data from four CLUSTER data files.  
; 
; Output variables: 
;   T       :  Hour of day as floating point array. 
;   B       :  Magnetic field data as structure of type CLVECTOR.
;   R       :  Spacecraft position data as structure of type CLVECTOR.
;   RBAR    :  Cluster barycenter as structure of type XYZVECTOR. 
;   DATABAD :  Flag to indicate data quality, type CLSCALAR_BYTE. 
;
; Keywords: 
;   FILENAMES   :  String array containing names of four data files. 
;   TBEG        :  Defines start time of data stream to be read. 
;   TEND        :  Defines end time of data stream to be read. 
;   DT          :  Sampling interval in seconds. 
;   TIME_FORMAT :  Options: 'HOUR_OF_DAY' (float), 'IGM_STRING'. 
; If keywords are not set, the user is prompted for input. 
; 
; The data files are expected to be in ASCII format.  
; 
; JoV, 4/01
; Modification history: 
;   6/01, JoV (with cdat version 0.2): 
;     * Keyword DT added. 
;   6/01, JoV (with cdat version 0.3): 
;     * Keyword TIME_FORMAT added, default is 'IGM_STRING'.
;     * TBEG and TEND are now strings.  
;     * DATABAD is of type { CLSCALAR_BYTE }. 
;   4/03, JoV (with cdat version 0.5): 
;     * Documentation update, no substantial change.  
;   6/03, JoV (with cdat version 0.52/0.62): 
;     * Replace call to obsolete routine str_sep by call to strsplit. 
;     * Remove switch khf_eof, effectively allowing for comments 
;       inside the data file (not just at the end). 
;     * Modification of array assignment procedure after reading 
;       of data (no longer using loop variable count but time 
;       information directly).  Should be more robust. 
; ===================================================================== 

; Check if keywords are set and prompt for input.
; -----------------------------------------------
; ... data files
  if not keyword_set(FILENAMES) then begin 
    FILENAMES = ' '
    FILENAMES = replicate( FILENAMES , 4 )
    dummy = ' ' 
    for i=0,3 do begin 
      print, 'Type in filename for Cluster satellite no. ', i+1
      read, dummy
      FILENAMES[i] = dummy
    endfor
  endif
; ... start time
  if not keyword_set(TBEG) then begin
    TBEG = ''
    print, 'Type in start time TBEG [hh:mm:ss] : '
    read, TBEG
  endif
; ... end time
  if not keyword_set(TEND) then begin
    TEND = ''
    print, 'Type in end time TEND [hh:mm:ss] : '
    read, TEND
  endif
;... sampling interval
  if not keyword_set(DT) then begin
    DT = 1.
    print, 'Type in sampling interval in seconds: '
    read, DT
  endif
;... time format 
  if not keyword_set(TIME_FORMAT) then TIME_FORMAT='IGM_STRING'

; Initialize data arrays etc
; --------------------------
; ... convert TBEG and TEND to floats
  reads, strsplit( TBEG , ':', /extract ), h, m, s
  tbegf = h + m/60. + s/3600.
  reads, strsplit( TEND , ':', /extract ), h, m, s
  tendf = h + m/60. + s/3600.
; ... data arrays
  nout = long( (tendf-tbegf)*(3600./DT) )
  T = tbegf + DT*dindgen(nout)/3600.
  B = replicate( { CLVECTOR } , nout )
  R = B
  RBAR = replicate( { XYZVECTOR } , nout )
  DATABAD = replicate( { CLSCALAR_BYTE } , nout )
  for i=0,3 do DATABAD[*].(i) = 1B
; ... dummy variables for read
  tdum = 0. 
  bxdum = 0. & bydum = 0. & bzdum = 0. 
  rxdum = 0. & rydum = 0. & rzdum = 0. 

; Open files, read data, assign values to DATABAD
; -----------------------------------------------
  for i = 0,3 do begin
    OPENR, /GET_LUN, unit, FILENAMES[i]
    print, 'File '+FILENAMES[i]+' opened for read.' 
    repeat begin 
      record = ''
      READF, unit, record
      record = strsplit(strcompress(strtrim(record,2)), ' ', /extract )
      case strupcase(TIME_FORMAT) of 
        'IGM_STRING': begin
          if ( strmid( record[0], 0, 3 )  eq '200' ) then begin 
            tdum = CONVERT_TIME( record[0], /TO_HOURS ) mod 24
            reads, record[1:6], bxdum, bydum, bzdum, rxdum, rydum, rzdum 
          endif
        end
        'HOUR_OF_DAY': begin
          if ( strpos( '0123456789-', strmid(record[0],0,1) ) ne -1 ) then $ 
            reads, record, tdum, bxdum, bydum, bzdum, rxdum, rydum, rzdum 
          end
        else: begin 
            print, '*** Error in PRO GETTBR: '
            print, '*** Value of keyword TIME_FORMAT not supported.'
            STOP
        end
      endcase
    endrep until ( eof(unit) or ( tdum ge tbegf ) )
    tfirstrecord = tbegf 
    if (tdum gt tbegf) then begin 
      tfirstrecord = tdum 
      T = T + (tfirstrecord - tbegf)
    endif
    itime = 0L
    while ( (not eof(unit)) and (itime le (nout-2L)) ) do begin 
      itime = fix( (tdum-tfirstrecord)*(3600./DT) + 0.5 ) 
      DATABAD[itime].(i) = 0B
      B[itime].x.(i) = bxdum
      B[itime].y.(i) = bydum
      B[itime].z.(i) = bzdum
      R[itime].x.(i) = rxdum
      R[itime].y.(i) = rydum
      R[itime].z.(i) = rzdum
      record = ''
      READF, unit, record
      record = strsplit( strcompress(strtrim(record,2)), ' ', /extract )
      case strupcase(TIME_FORMAT) of 
        'IGM_STRING': begin
          if ( strmid( record[0], 0, 3 )  eq '200' ) then begin 
             tdum = CONVERT_TIME( record[0], /TO_HOURS ) mod 24
             reads, record[1:6], bxdum, bydum, bzdum, rxdum, rydum, rzdum 
          endif
        end
        'HOUR_OF_DAY': begin 
          if ( strpos( '0123456789-', strmid(record[0],0,1) ) ne -1 ) then $ 
             reads, record, tdum, bxdum, bydum, bzdum, rxdum, rydum, rzdum 
        end
        else: begin 
             print, '*** Error in PRO GETTBR: '
             print, '*** Value of keyword TIME_FORMAT not supported.'
             STOP
        end
      endcase
    endwhile
    CLOSE,unit
  endfor
  print, ' '

; Compute average position (i.e., pos. of barycentre).
; ---------------------------------------------------- 
  RBAR.x = 0.25*( R.x.c1 + R.x.c2 + R.x.c3 + R.x.c4 )
  RBAR.y = 0.25*( R.y.c1 + R.y.c2 + R.y.c3 + R.y.c4 )
  RBAR.z = 0.25*( R.z.c1 + R.z.c2 + R.z.c3 + R.z.c4 )

  END
