
  PRO GETTBR_MOD, T, B, R, RBAR, DATABAD,                        $
              filenames=FILENAMES, tbeg=TBEG, tend=TEND, dt=DT,  $
              time_format=TIME_FORMAT
; =====================================================================
; GETTBR reads data magnetic field data from four CLUSTER data files.  
;
; ==> This is a modified version, called "GETTBR_MOD": mfo, 25 Feb 2004
;     It reads all 4 data files and interpolates the values to a common
;     equidistant time array, avoiding different phases and spin times.
; 
; Output variables: 
;   T       :  Hour of day as floating point array. 
;   B       :  Magnetic field data as structure of type CLVECTOR.
;   R       :  Spacecraft position data as structure of type CLVECTOR.
;   RBAR    :  Cluster barycenter as structure of type XYZVECTOR. 
;   DATABAD :  Flag to indicate data quality, type CLSCALAR_BYTE. 
;
; Keywords: 
;   FILENAMES   :  String array containing names of four data files. 
;   TBEG        :  Defines start time of data stream to be read. 
;   TEND        :  Defines end time of data stream to be read. 
;   DT          :  Sampling interval in seconds. 
;   TIME_FORMAT :  Options: 'HOUR_OF_DAY' (float), 'IGM_STRING'. 
; If keywords are not set, the user is prompted for input. 
; 
; The data files are expected to be in ASCII format.  
; 
; JoV, 4/01
; Modification history: 
;   6/01, JoV (with cdat version 0.2): 
;     * Keyword DT added. 
;   6/01, JoV (with cdat version 0.3): 
;     * Keyword TIME_FORMAT added, default is 'IGM_STRING'.
;     * TBEG and TEND are now strings.  
;     * DATABAD is of type { CLSCALAR_BYTE }. 
;   4/03, JoV (with cdat version 0.5): 
;     * Documentation update, no substantial change.  
;   6/03, JoV (with cdat version 0.52/0.62): 
;     * Replace call to obsolete routine str_sep by call to strsplit. 
;     * Remove switch khf_eof, effectively allowing for comments 
;       inside the data file (not just at the end). 
;     * Modification of array assignment procedure after reading 
;       of data (no longer using loop variable count but time 
;       information directly).  Should be more robust. 
; ===================================================================== 

; Check if keywords are set and prompt for input.
; -----------------------------------------------
; ... data files
  if not keyword_set(FILENAMES) then begin 
    FILENAMES = ' '
    FILENAMES = replicate( FILENAMES , 4 )
    dummy = ' ' 
    for i=0,3 do begin 
      print, 'Type in filename for Cluster satellite no. ', i+1
      read, dummy
      FILENAMES[i] = dummy
    endfor
  endif
; ... start time
  if not keyword_set(TBEG) then begin
    TBEG = ''
    print, 'Type in start time TBEG [hh:mm:ss] : '
    read, TBEG
  endif
; ... end time
  if not keyword_set(TEND) then begin
    TEND = ''
    print, 'Type in end time TEND [hh:mm:ss] : '
    read, TEND
  endif
;... sampling interval
  if not keyword_set(DT) then begin
    DT = 1.
    print, 'Type in sampling interval in seconds: '
    read, DT
  endif
;... time format 
  if not keyword_set(TIME_FORMAT) then TIME_FORMAT='IGM_STRING'

; Initialize data arrays etc
; --------------------------
; ... convert TBEG and TEND to floats
  reads, strsplit( TBEG , ':', /extract ), h, m, s
  tbegf = h + m/60. + s/3600.
  reads, strsplit( TEND , ':', /extract ), h, m, s
  tendf = h + m/60. + s/3600.
; ... data arrays
;;nout = long( (tendf-tbegf)*(3600./DT) )
  nout = long( (tendf-tbegf)*(3600./DT) ) + 1L ; mfo, 29 Jul 2003
  T    = tbegf + DT*dindgen(nout)/3600.
  B    = replicate({ CLVECTOR }, nout)
  R    = B
  RBAR = replicate({ XYZVECTOR }, nout)
  DATABAD = replicate({ CLSCALAR_BYTE }, nout)
  for i=0,3 do DATABAD[*].(i) = 1B
; ... dummy variables for read
  tdum = 0. 
  bxdum = 0. & bydum = 0. & bzdum = 0. 
  rxdum = 0. & rydum = 0. & rzdum = 0. 

; Open files, read data, assign values to DATABAD
; -----------------------------------------------
  for i = 0,3 do begin
    spawn,['wc','-lw',filenames[i]],flen,/noshell
    if strlen(flen) LE 0 then begin
      print,'*** Error in PRO GETTBR_MOD:'
      print,'*** Input file No. '+string(i+1,f="(I1)")+' missing: '+filenames[i]
      STOP
    endif
    flen= strsplit(flen,/extract)
    nrl = LONG(flen[0])-2L ; two lines of comments at the end of the file
    nrc = ROUND(float(flen[1])/float(nrl))
    if nrc NE 7 then begin
      print,'*** Error in PRO GETTBR_MOD:'
      print,'*** Unexpected number of columns in input file '+filenames[i]
      STOP
    endif
    T_read  = replicate( 0.0, nrl)
    B_read  = replicate({XYZVECTOR},nrl)
    R_read  = replicate({XYZVECTOR},nrl)
    n       = 0L
    OPENR, /GET_LUN, unit, FILENAMES[i]
    print, 'File '+FILENAMES[i]+' opened for read.'
    repeat begin 
      record = ''
      READF, unit, record
      record = strsplit(strcompress(strtrim(record,2)), ' ', /extract )
      tdum_p = tdum ; added to guarantee ascending time order: mfo 09 Mar 2004
      case strupcase(TIME_FORMAT) of 
        'IGM_STRING': begin
          if ( strmid( record[0], 0, 3 )  eq '200' ) then begin 
            tdum = CONVERT_TIME( record[0], /TO_HOURS ) mod 24
            reads, record[1:6], bxdum, bydum, bzdum, rxdum, rydum, rzdum 
          endif
        end
        'HOUR_OF_DAY': begin
          if ( strpos( '0123456789-', strmid(record[0],0,1) ) ne -1 ) then $ 
            reads, record, tdum, bxdum, bydum, bzdum, rxdum, rydum, rzdum 
          end
        else: begin 
            print, '*** Error in PRO GETTBR: '
            print, '*** Value of keyword TIME_FORMAT not supported.'
            STOP
        end
      endcase
      while (tdum LT tdum_p) do tdum = tdum + 24.          ; mfo 09 Mar 2004
      T_read[n]   = tdum
      B_read[n].x = bxdum
      B_read[n].y = bydum
      B_read[n].z = bzdum
      R_read[n].x = rxdum
      R_read[n].y = rydum
      R_read[n].z = rzdum
      n           = n + 1L
    endrep until ( eof(unit) OR ( n GE nrl ) )
    while (median(T_read) GT 24.) do T_read = T_read - 24. ; mfo 09 Mar 2004
    for it = 0,nout-1 do begin
      ii = where(abs(T_read-T[it])*3600. LE DT)
      if ii[0] GE 0 then DATABAD[it].(i) = 0B
    endfor
    B.x.(i) = interpol(B_read.x,T_read,T)
    B.y.(i) = interpol(B_read.y,T_read,T)
    B.z.(i) = interpol(B_read.z,T_read,T)
    R.x.(i) = interpol(R_read.x,T_read,T)
    R.y.(i) = interpol(R_read.y,T_read,T)
    R.z.(i) = interpol(R_read.z,T_read,T)
    CLOSE,unit
  endfor
  print, ' '

; Compute average position (i.e., pos. of barycentre).
; ---------------------------------------------------- 
  RBAR.x = 0.25*( R.x.c1 + R.x.c2 + R.x.c3 + R.x.c4 )
  RBAR.y = 0.25*( R.y.c1 + R.y.c2 + R.y.c3 + R.y.c4 )
  RBAR.z = 0.25*( R.z.c1 + R.z.c2 + R.z.c3 + R.z.c4 )

  END
