
  PRO PLOT_CDAT1, T, CURLB, DIVB, RPARAM, KPARAM,    $ 
      width_smo=WIDTH_SMO, conv_factor=CONV_FACTOR,  $
      psfile=PSFILE, infostring=INFOSTRING,          $
      xtstring=XTSTRING, ytstring=YTSTRING
; ====================================================================
; PLOT various time series for routine CDAT: 
; -- curlB, 
; -- divB, 
; -- RPARAM and 1/KPARAM, 
; -- divB/KPARAM as an accuracy estimate. 
; 
; Usage: only inside routine cdat.
; 
; Input variables: 
;   T      :  Hour of day as floating point array. 
;   CURLB  :  Structure of type XYZVECTOR containing curl of B. 
;   DIVB   :  Floating point array containing div of B.
;   RPARAM :  Square root of trace of position tensor. 
;   KPARAM :  Square root of trace of reciprocal tensor. 
;
; Keywords: 
;   WIDTH_SMO    :  If set, smoothed arrays are plotted using 
;                   parameter WIDTH_SMO in SMOOTH(...).  
;   CONV_FACTOR  :  If set, results are multiplied by CONV_FACTOR. 
;   PSFILE       :  If set, then graphics output is to file PSFILE. 
;                   Default graphics output is to screen ('X'). 
;   INFOSTRING   :  String to be written at the lower right edge. 
;   XTSTRING     :  x-title.
;   YTSTRING     :  y-title: units of divB and curlB. 
; 
; JoV, 5/01.
; Modification history: 
;   8/01, JoV : 
;     * Bug fix for WIDTH_SMO=1. 
;     * Bug fix for YTSTRING. 
;   4/03, JoV (with cdat version 0.5): 
;     * UT on x-axis in hh:mm.  
;     * Reset of all plot parameters. 
;     * Documentation update. 
; ====================================================================

; Check keyword settings.
; -----------------------
  if not keyword_set(WIDTH_SMO) then WIDTH_SMO=1
  wsmo = WIDTH_SMO
  if not keyword_set(CONV_FACTOR) then CONV_FACTOR=1
  if not keyword_set(XTSTRING) then XTSTRING='X'
  if not keyword_set(YTSTRING) then YTSTRING='Y'

; Plot parameters.
; ----------------
  !P.MULTI=[0,1,6,0,1]
  ntime = n_elements(T) 
  default_xrange = !X.RANGE
  !X.RANGE = [ T[0] , T[ntime-1] ]*60.D0
  default_xstyle = !X.STYLE
  !X.STYLE = 1

; Switch to PS device if necessary.
; ---------------------------------
  if keyword_set(PSFILE) then begin
    device_name = !D.NAME
    SET_PLOT, 'PS'
    DEVICE, xoff=2, yoff=2, xsize=16, ysize=25, FILENAME=PSFILE, $
            landscape=0
  endif 

; Compute smooth curlB and plot.
; ------------------------------
  if (wsmo ge 3) then begin 
    curlBx_smo = CONV_FACTOR*SMOOTH( CURLB.x , wsmo )
    curlBy_smo = CONV_FACTOR*SMOOTH( CURLB.y , wsmo )
    curlBz_smo = CONV_FACTOR*SMOOTH( CURLB.z , wsmo )
  endif else begin 
    curlBx_smo = CONV_FACTOR * CURLB.x
    curlBy_smo = CONV_FACTOR * CURLB.y 
    curlBz_smo = CONV_FACTOR * CURLB.z 
  endelse
  curlBx_smo = curlBx_smo[ wsmo : ntime-wsmo-1 ]
  curlBy_smo = curlBy_smo[ wsmo : ntime-wsmo-1 ]
  curlBz_smo = curlBz_smo[ wsmo : ntime-wsmo-1 ]
  tmin_smo = T[ wsmo : ntime-wsmo-1 ]*60.D0
  PLOT, tmin_smo, curlBx_smo, title='!6Curl B estimation: x-comp.',   $
        xtitle=XTSTRING, ytitle='!6curlB_x '+ytstring, xtickformat='HHMM'
  PLOT, tmin_smo, curlBy_smo, title='!6Curl B estimation: y-comp.',   $
        xtitle=XTSTRING, ytitle='!6curlB_y '+ytstring, xtickformat='HHMM'
  PLOT, tmin_smo, curlBz_smo, title='!6Curl B estimation: z-comp.',   $
        xtitle=XTSTRING, ytitle='!6curlB_z '+ytstring, xtickformat='HHMM'

; Compute smooth divB and plot.
; -----------------------------
  if (wsmo ge 3) then begin 
    divB_smo = CONV_FACTOR*SMOOTH( divB , wsmo ) 
  endif else begin
    divB_smo = CONV_FACTOR * divB 
  endelse
  divB_smo = divB_smo[ wsmo : ntime-wsmo-1 ]
  PLOT, tmin_smo, divB_smo, title='!6Div B estimation',               $
        xtitle=xtstring, ytitle='!6divB '+ytstring, xtickformat='HHMM'

; Quality parameters and error estimate.
; --------------------------------------
  tmin = T*60.D0
  PLOT, tmin, RPARAM/(sqrt(3)*CONV_FACTOR),                       $
        title='!6Geometric quality parameters',                   $
        xtitle=xtstring, ytitle='!6R/!9R!63, !9R!63/K [Mm]',      $
        xtickformat='HHMM'
  OPLOT, tmin, (sqrt(3)/CONV_FACTOR)/KPARAM, linestyle=2 
  kp_smo = KPARAM[ wsmo : ntime-wsmo-1 ]
  PLOT, tmin_smo, divB_smo/(kp_smo*CONV_FACTOR),                  $
        title='Accuracy estimate', xtitle=xtstring,               $
        ytitle='!6divB/K [nT]', xtickformat='HHMM'

; Write INFOSTRING on lower right edge.
; -------------------------------------
  if keyword_set(INFOSTRING) then                              $
      XYOUTS, 1,0, INFOSTRING, /normal, alignment=1, charsize=0.5  

; Switch back to X device and reset other plot parameters.
; --------------------------------------------------------
  if keyword_set(PSFILE) then begin
     DEVICE, /close
     SET_PLOT, device_name
  endif 
  !X.RANGE = default_xrange 
  !X.STYLE = default_xstyle
  !P.MULTI = 0

  END
