
  PRO PLOT_CDAT2, T, CURLB, DIVB, width_smo=WIDTH_SMO,        $
      psfile=PSFILE, infostring=INFOSTRING, xtstring=XTSTRING
; ===============================================================
; PLOT various time series for routine CDAT: 
; -- current density curlB/mu0, 
; -- divB/mu0. 
; T is expected to be the hour of day and will be annotated 
; at the x-axis as HH:MM. 
; 
; Usage: only inside routine cdat.
; 
; Input variables: 
;   T      :  Hour of day as floating point array. 
;   CURLB  :  Structure of type XYZVECTOR containing curl of B. 
;   DIVB   :  Floating point array containing div of B.
;
; Keywords: 
;   WIDTH_SMO    :  If set, smoothed arrays are plotted using 
;                   parameter WIDTH_SMO in SMOOTH(...).  
;   PSFILE       :  If set, then graphics output is to file PSFILE. 
;                   Default graphics output is to screen ('X'). 
;   INFOSTRING   :  String to be written at the lower right edge. 
;   XTSTRING     :  x-title.
; 
; JoV, 8/01.  Modified from PRO PLOT_CDAT1. 
; Modification history: 
;   4/03, JoV (with cdat version 0.5): 
;     * Reset of all plot parameters. 
;     * Documentation update. 
;     * Portrait instead of landscape. 
; ===============================================================

; Check keyword settings.
; -----------------------
  if not keyword_set(WIDTH_SMO) then WIDTH_SMO=1
  wsmo = WIDTH_SMO
  if not keyword_set(XTSTRING) then XTSTRING='X'
  if not keyword_set(YTSTRING) then YTSTRING='Y'

; Default values.
; ---------------
  CONV_FACTOR = 10./(4.*!PI)
  YTSTRING = '!6[!S!Dm!E2!R!S-!R!U!7l!6A!N]'

; Plot parameters.
; ----------------
  !P.MULTI=[0,1,4,0,1]
  ntime = n_elements(T) 
  default_xrange = !X.RANGE
  !X.RANGE = [ T[0] , T[ntime-1] ]*60.D0
  default_xstyle = !X.STYLE
  !X.STYLE = 1
  default_pcharsize = !P.CHARSIZE 
  !P.CHARSIZE = 2.

; Switch to PS device if necessary.
; ---------------------------------
  if keyword_set(PSFILE) then begin
    default_dname = !D.NAME
    SET_PLOT, 'PS'
    DEVICE, xoff=2, yoff=2, xsize=16, ysize=25, FILENAME=PSFILE, $
            landscape=0
  endif 

; Compute smooth curlB and plot.
; ------------------------------
  if (wsmo ge 3) then begin 
    curlBx_smo = CONV_FACTOR*SMOOTH( CURLB.x , wsmo )
    curlBy_smo = CONV_FACTOR*SMOOTH( CURLB.y , wsmo )
    curlBz_smo = CONV_FACTOR*SMOOTH( CURLB.z , wsmo )
  endif else begin 
    curlBx_smo = CONV_FACTOR * CURLB.x 
    curlBy_smo = CONV_FACTOR * CURLB.y 
    curlBz_smo = CONV_FACTOR * CURLB.z 
  endelse
  curlBx_smo = curlBx_smo[ wsmo : ntime-wsmo-1 ]
  curlBy_smo = curlBy_smo[ wsmo : ntime-wsmo-1 ]
  curlBz_smo = curlBz_smo[ wsmo : ntime-wsmo-1 ]
  tmin_smo = T[ wsmo : ntime-wsmo-1 ]*60.D0
  PLOT, tmin_smo, curlBx_smo, title='!6Curl B estimation: x-comp.', $
        ytitle='!6curl!17B!6!Bx!N/!7l!6!D0!N '+ytstring, xtickformat='HHMM'
  PLOT, tmin_smo, curlBy_smo, title='!6Curl B estimation: y-comp.', $
        ytitle='!6curl!17B!6!By!N/!7l!6!D0!N '+ytstring, xtickformat='HHMM'
  PLOT, tmin_smo, curlBz_smo, title='!6Curl B estimation: z-comp.', $
        ytitle='!6curl!17B!6!Bz!N/!7l!6!D0!N '+ytstring, xtickformat='HHMM'

; Compute smooth divB and plot.
; -----------------------------
  if (wsmo ge 3) then begin 
    divB_smo = CONV_FACTOR*SMOOTH( divB , wsmo ) 
  endif else begin
    divB_smo = CONV_FACTOR * divB  
  endelse
  divB_smo = divB_smo[ wsmo : ntime-wsmo-1 ]
  PLOT, tmin_smo, divB_smo, xtitle=xtstring, title='!6Div B estimation',  $
        ytitle='!6div!17B/!7l!6!D0!N '+ytstring, xtickformat='HHMM'

; Write INFOSTRING on lower right edge.
; -------------------------------------
  if keyword_set(INFOSTRING) then                              $
      XYOUTS, 1,0, INFOSTRING, /normal, alignment=1, charsize=0.5  

; Switch back to X device and reset other plot parameters. 
; --------------------------------------------------------
  if keyword_set(PSFILE) then begin
     DEVICE, /close
     SET_PLOT, default_dname
  endif 
  !X.RANGE = default_xrange 
  !X.STYLE = default_xstyle
  !P.CHARSIZE = default_pcharsize
  !P.MULTI = 0

  END
